import Gio from 'gi://Gio';
import Gtk from 'gi://Gtk';
import GObject from 'gi://GObject';
import Adw from 'gi://Adw';
import { ExtensionPreferences } from 'resource:///org/gnome/Shell/Extensions/js/extensions/prefs.js';

// GENERATED CODE: DO NOT EDIT
const ANIMATIONS_ENABLED = "animations-enabled";
const DEBUG = "debug";
const MOVE_FOCUSED_DOWN = "move-focused-down";
const MOVE_FOCUSED_LEFT = "move-focused-left";
const MOVE_FOCUSED_RIGHT = "move-focused-right";
const MOVE_FOCUSED_UP = "move-focused-up";
const MOVERESIZE_ENABLED = "moveresize-enabled";
const PRESET_RESIZE_1 = "preset-resize-1";
const PRESET_RESIZE_10 = "preset-resize-10";
const PRESET_RESIZE_11 = "preset-resize-11";
const PRESET_RESIZE_12 = "preset-resize-12";
const PRESET_RESIZE_2 = "preset-resize-2";
const PRESET_RESIZE_3 = "preset-resize-3";
const PRESET_RESIZE_4 = "preset-resize-4";
const PRESET_RESIZE_5 = "preset-resize-5";
const PRESET_RESIZE_6 = "preset-resize-6";
const PRESET_RESIZE_7 = "preset-resize-7";
const PRESET_RESIZE_8 = "preset-resize-8";
const PRESET_RESIZE_9 = "preset-resize-9";
const PREVENT_SNAPPING = "prevent-snapping";
const SHOW_ICON = "show-icon";
const SHOW_TABS = "show-tabs";
const SPAN_MULTIPLE_ZONES = "span-multiple-zones";
const USE_MODIFIER = "use-modifier";
const WINDOW_MARGIN = "window-margin";

// @ts-ignore
// Globals
const pretty_names = {
    [PRESET_RESIZE_1]: 'Layout 1',
    [PRESET_RESIZE_2]: 'Layout 2',
    [PRESET_RESIZE_3]: 'Layout 3',
    [PRESET_RESIZE_4]: 'Layout 4',
    [PRESET_RESIZE_5]: 'Layout 5',
    [PRESET_RESIZE_6]: 'Layout 6',
    [PRESET_RESIZE_7]: 'Layout 7',
    [PRESET_RESIZE_8]: 'Layout 8',
    [PRESET_RESIZE_9]: 'Layout 9',
    [PRESET_RESIZE_10]: 'Layout 10',
    [PRESET_RESIZE_11]: 'Layout 11',
    [PRESET_RESIZE_12]: 'Layout 12',
    [MOVE_FOCUSED_UP]: 'Move focused window up',
    [MOVE_FOCUSED_DOWN]: 'Move focused window down',
    [MOVE_FOCUSED_LEFT]: 'Move focused window left',
    [MOVE_FOCUSED_RIGHT]: 'Move focused window right',
};
class GSnapPreferences extends ExtensionPreferences {
    fillPreferencesWindow(window) {
        this.settings = super.getSettings();
        window._settings = this.settings;
        const preferencesPage = new Adw.PreferencesPage({
            title: "Preferences",
            icon_name: "emblem-system-symbolic"
        });
        preferencesPage.add(this.basics());
        preferencesPage.add(this.miscSettings());
        window.add(preferencesPage);
        const shortcutsPage = new Adw.PreferencesPage({
            title: "Shortcuts",
            icon_name: "preferences-desktop-keyboard-symbolic"
        });
        shortcutsPage.add(this.shortcuts());
        window.add(shortcutsPage);
        const aboutPage = new Adw.PreferencesPage({
            title: "About",
            icon_name: "help-about-symbolic"
        });
        aboutPage.add(this.about());
        aboutPage.add(this.aboutLinks());
        window.add(aboutPage);
    }
    basics() {
        const group = new Adw.PreferencesGroup({
            title: 'Basics'
        });
        this.add_check(group, SHOW_ICON, "Show icon");
        const show_tabs_check = this.add_check(group, SHOW_TABS, "Show tabs", "This feature is not supported if you have the \"Hold ALT to span multiple zones\" feature enabled");
        this.add_check(group, MOVERESIZE_ENABLED, "Enable accelerators for moving and resizing windows");
        this.add_check(group, USE_MODIFIER, "Hold CTRL to snap windows");
        const span_multiple_zones_check = this.add_check(group, SPAN_MULTIPLE_ZONES, "Hold ALT to span multiple zones", "This feature is not supported if you have the \"Show tabs\" feature enabled");
        this.add_check(group, PREVENT_SNAPPING, "Hold Super to prevent snapping windows");
        // disable "Span multiple zones" setting if "Show tabs" setting was already enabled
        if (this.settings.get_boolean(SHOW_TABS)) {
            span_multiple_zones_check.set_sensitive(false);
            // disable "Show tabs" setting if "Span multiple zones" setting was already enabled
        }
        else if (this.settings.get_boolean(SPAN_MULTIPLE_ZONES)) {
            show_tabs_check.set_sensitive(false);
        }
        // Watch for changes to the setting SETTINGS.SHOW_TABS
        this.settings.connect(`changed::${SHOW_TABS}`, (settings, changed_key) => {
            // disable "Span multiple zones" setting if "Show tabs" setting is enabled by the user
            span_multiple_zones_check.set_sensitive(!settings.get_boolean(changed_key));
        });
        // Watch for changes to the setting SETTINGS.SPAN_MULTIPLE_ZONES
        this.settings.connect(`changed::${SPAN_MULTIPLE_ZONES}`, (settings, changed_key) => {
            // disable "Show tabs" setting if "Span multiple zones" setting is enabled by the user
            show_tabs_check.set_sensitive(!settings.get_boolean(changed_key));
        });
        this.add_check(group, ANIMATIONS_ENABLED, "Enable animations");
        this.add_int(group, WINDOW_MARGIN, "Window margin", 0, 32, 1, "Window margins and invisible borders around screen.");
        return group;
    }
    miscSettings() {
        const group = new Adw.PreferencesGroup({
            title: 'Miscellaneous'
        });
        this.add_check(group, DEBUG, "Debug", "To see debug messages, in terminal run journalctl /usr/bin/gnome-shell -f");
        return group;
    }
    shortcuts() {
        const group = new Adw.PreferencesGroup({
            title: 'Shortcuts'
        });
        let model = new Gtk.ListStore();
        model.set_column_types([
            GObject.TYPE_STRING,
            GObject.TYPE_STRING,
            GObject.TYPE_INT,
            GObject.TYPE_INT
        ]);
        for (let key in pretty_names) {
            this.add_hotkey(model, key, pretty_names[key]);
        }
        const titleRenderer = new Gtk.CellRendererText();
        const titleColumn = new Gtk.TreeViewColumn({
            title: 'Shortcut',
            expand: true
        });
        titleColumn.pack_start(titleRenderer, true);
        titleColumn.add_attribute(titleRenderer, 'text', 1);
        const keybindingsCellRenderer = new Gtk.CellRendererAccel({
            editable: true,
            'accel-mode': Gtk.CellRendererAccelMode.GTK
        });
        const _settings = this.settings;
        keybindingsCellRenderer.connect('accel-cleared', function (_rend, str_iter) {
            let [success, iter] = model.get_iter_from_string(str_iter);
            if (!success) {
                throw new Error("Something be broken, yo.");
            }
            let name = model.get_value(iter, 0);
            model.set(iter, [3], [0]);
            _settings.set_strv(name, ['']);
        });
        keybindingsCellRenderer.connect('accel-edited', function (rend, str_iter, key, mods) {
            let value = Gtk.accelerator_name(key, mods);
            let [success, iter] = model.get_iter_from_string(str_iter);
            if (!success) {
                throw new Error("Something be broken, yo.");
            }
            let name = model.get_value(iter, 0);
            model.set(iter, [2, 3], [mods, key]);
            _settings.set_strv(name, [value]);
        });
        const keybindingsColumn = new Gtk.TreeViewColumn({
            title: 'Keybindings'
        });
        keybindingsColumn.pack_end(keybindingsCellRenderer, false);
        keybindingsColumn.add_attribute(keybindingsCellRenderer, 'accel-mods', 2);
        keybindingsColumn.add_attribute(keybindingsCellRenderer, 'accel-key', 3);
        const treeview = new Gtk.TreeView({
            model,
            hexpand: true
        });
        treeview.append_column(titleColumn);
        treeview.append_column(keybindingsColumn);
        group.add(treeview);
        return group;
    }
    about() {
        const group = new Adw.PreferencesGroup();
        var logo = new Gtk.Image({
            width_request: 150,
            height_request: 150,
        });
        logo.set_from_file(`${super.path}/images/icon.png`);
        const nameLabel = new Gtk.Label({
            label: `<b>gSnap</b>`,
            xalign: .5,
            margin_top: 10,
            margin_bottom: 5,
            hexpand: true,
            halign: Gtk.Align.CENTER,
            use_markup: true
        });
        const descriptionLabel = new Gtk.Label({
            label: `Organize windows in customizable snap zones like FancyZones on Windows.`,
            xalign: .5,
            margin_bottom: 20,
            hexpand: true,
            halign: Gtk.Align.CENTER,
            use_markup: true
        });
        group.add(logo);
        group.add(nameLabel);
        group.add(descriptionLabel);
        return group;
    }
    aboutLinks() {
        const group = new Adw.PreferencesGroup({
            title: 'Useful links'
        });
        this.add_linkbutton(group, 'Contribute to the project', 'https://github.com/GnomeSnapExtensions/gSnap');
        this.add_linkbutton(group, 'Report a bug', 'https://github.com/GnomeSnapExtensions/gSnap/issues');
        return group;
    }
    add_check(group, setting, title, subtitle = null) {
        const toggle = new Gtk.Switch({
            active: this.settings.get_boolean(setting),
            valign: Gtk.Align.CENTER,
        });
        this.settings.bind(setting, toggle, 'active', Gio.SettingsBindFlags.DEFAULT);
        const row = new Adw.ActionRow({
            title,
            subtitle,
            activatable_widget: toggle,
        });
        row.add_suffix(toggle);
        group.add(row);
        return toggle;
    }
    add_int(group, setting, title, lower, upper, step_increment, subtitle = null) {
        const spin = Gtk.SpinButton.new_with_range(lower, upper, step_increment);
        spin.set_valign(Gtk.Align.CENTER);
        this.settings.bind(setting, spin.get_adjustment(), 'value', Gio.SettingsBindFlags.DEFAULT);
        const row = new Adw.ActionRow({
            title,
            subtitle,
            activatableWidget: spin
        });
        row.add_suffix(spin);
        group.add(row);
    }
    add_linkbutton(group, title, uri) {
        let button = new Gtk.LinkButton({
            uri: uri,
            halign: Gtk.Align.CENTER,
            valign: Gtk.Align.CENTER,
        });
        const row = new Adw.ActionRow({
            title,
            activatable_widget: button,
        });
        row.add_suffix(button);
        group.add(row);
    }
    add_hotkey(model, name, pretty_name) {
        // ignore ok as failure treated as disabled
        const [_ok, key, mods] = Gtk.accelerator_parse(this.settings.get_strv(name)[0]);
        let row = model.insert(-1);
        model.set(row, [0, 1, 2, 3], [name, pretty_name, mods, key]);
    }
}

export { GSnapPreferences as default };
